@@ -38,6 +38,22 @@ class DefaultVpcAlreadyExists(EC2ClientError):
         )
 
 
+class DefaultVpcDoesNotExistError(EC2ClientError):
+    def __init__(self) -> None:
+        super().__init__(
+            "DefaultVpcDoesNotExist",
+            "No default VPC exists for this account in this region.",
+        )
+
+
+class DefaultSubnetAlreadyExistsInAvailabilityZoneError(EC2ClientError):
+    def __init__(self, subnet_id: str, availability_zone: str) -> None:
+        super().__init__(
+            "DefaultSubnetAlreadyExistsInAvailabilityZone",
+            f"'{subnet_id}' is already the default subnet in {availability_zone}.",
+        )
+
+
 class DependencyViolationError(EC2ClientError):
     def __init__(self, message: str):
         super().__init__("DependencyViolation", message)
@@ -197,6 +213,14 @@ class InvalidSecurityGroupNotFoundError(EC2ClientError):
         )
 
 
+class InvalidSecurityGroupRuleIdNotFoundError(EC2ClientError):
+    def __init__(self, name: Any):
+        super().__init__(
+            "InvalidSecurityGroupRuleId.NotFound",
+            f"The security group rule ID '{name}' does not exist",
+        )
+
+
 class InvalidPermissionNotFoundError(EC2ClientError):
     def __init__(self) -> None:
         super().__init__(
@@ -547,7 +571,7 @@ class RulesPerSecurityGroupLimitExceededError(EC2ClientError):
     def __init__(self) -> None:
         super().__init__(
             "RulesPerSecurityGroupLimitExceeded",
-            "The maximum number of rules per security group " "has been reached.",
+            "The maximum number of rules per security group has been reached.",
         )
 
 
@@ -689,6 +713,14 @@ class OperationNotPermitted4(EC2ClientError):
         )
 
 
+class OperationDisableApiStopNotPermitted(EC2ClientError):
+    def __init__(self, instance_id: str):
+        super().__init__(
+            "OperationNotPermitted",
+            f"The instance '{instance_id}' may not be terminated. Modify its 'disableApiStop' instance attribute and try again.",
+        )
+
+
 # Raised when attempting to accept or reject a VPC peering connection request for a VPC not belonging to self
 class OperationNotPermitted5(EC2ClientError):
     def __init__(self, account_id: str, pcx_id: str, operation: str):
